# SeqGeq / Seurat analysis

# Script modified from Satija lab's work for the R Seurat package.
# Script modified by Miguel Velazquez and Ian Taylor at FlowJo 
# For use w/ SeqGeq 
# Seurat is developed and maintained by the Satija lab. 
# Code is  released in GPL (3.0) license. 

#Disable error messaging in v1.0, prevent early exit.
options(warn=-1)


library(Seurat)
library(Matrix)
library(plyr)

writeLines("Import CSV target file")
# Read CSV from SeqGeq
importedCSVfile <- read.csv(file= "SG_TARGET_DATA_PATH", sep = ",", row.names = 1, header = TRUE, check.names = FALSE)

writeLines("Import CSV sample file")

# Some data has numbers as cellIDs only, append a string to cellIds, remove on export.
rownames(importedCSVfile) <- paste("AppID", rownames(importedCSVfile),sep="_")
importedCSVfile = t(importedCSVfile)
# Read CSV from SeqGeq
importedCSVfile2 <- read.csv(file= "SG_SAMPLE_DATA_PATH", sep = ",", row.names = 1, header = TRUE, check.names = FALSE)

# Some data has numbers as cellIDs only, append a string to cellIds, remove on export.
rownames(importedCSVfile2) <- paste("AttID", rownames(importedCSVfile2),sep="_")
importedCSVfile2 = t(importedCSVfile2)

# Set output directory
outfileDirectory = "SG_OUTPUT_DIRECTORY_PATH"

# Import parameters to keep
if(SG_IMPORT_KEEP_TARGET)
{
    drop1 <- read.csv(file= paste0("SG_OUTPUT_DIRECTORY_PATH", "SG_SEURAT_RUN_ID_keepTargetParameters.csv"), header = FALSE, check.names = FALSE)
    keep1 =as.data.frame(rownames(importedCSVfile) %in% drop1$V1)
    targetKeepDF = importedCSVfile[unlist(keep1),]
    importedCSVfile = importedCSVfile[!unlist(keep1),]


}
if(SG_IMPORT_KEEP_SAMPLE)
{
    drop <- read.csv(file= paste0("SG_OUTPUT_DIRECTORY_PATH", "SG_SEURAT_RUN_ID_keepSampleParameters.csv"), header = FALSE, check.names = FALSE)
    keep =as.data.frame(rownames(importedCSVfile2) %in% drop$V1)
    sampleKeepDF = importedCSVfile2[unlist(keep),]
    importedCSVfile2 = importedCSVfile2[!unlist(keep),]

}


writeLines("Create Seurat alignment objects")

seqwell <- CreateSeuratObject(raw.data = importedCSVfile)
seqwell <- NormalizeData(object = seqwell)
seqwell <- ScaleData(object = seqwell)
seqwell <- FindVariableGenes(object = seqwell, do.plot = FALSE)

tenx <- CreateSeuratObject(raw.data = importedCSVfile2)
tenx <- NormalizeData(object = tenx)
tenx <- ScaleData(object = tenx)
tenx <- FindVariableGenes(object = tenx, do.plot = FALSE)

# we will take the union of the top 2k variable genes in each dataset for
# alignment note that we use 1k genes in the manuscript examples, you can
# try this here with negligible changes to the overall results
writeLines("Take SG_UNION_GENES intersection for alignment")

hvg.seqwell <- rownames(x = head(x = seqwell@hvg.info, n = SG_UNION_GENES))
hvg.tenx <- rownames(x = head(x = tenx@hvg.info, n = SG_UNION_GENES))
hvg.union <- union(x = hvg.seqwell, y = hvg.tenx)

# lastly, we set the 'protocol' in each dataset for easy identification
# later it will be transferred to the merged object in RunCCA
tenx@meta.data[, "protocol"] <- "Sample"
seqwell@meta.data[, "protocol"] <- "Target"
writeLines("Start Alignment")
pbmc <- RunCCA(object = tenx, object2 = seqwell, genes.use = hvg.union)


cells1 = ncol(importedCSVfile)
cells2 = ncol(importedCSVfile2)

if(cells1>cells2){

numCells = cells2
}else{

numCells = cells1

}   

# visualize results of CCA plot CC1 versus CC2 and look at a violin plot
p1 <- DimPlot(object = pbmc, reduction.use = "cca", group.by = "protocol", pt.size = 0.5, 
    do.return = TRUE)
p2 <- VlnPlot(object = pbmc, features.plot = "CC1", group.by = "protocol", do.return = TRUE)

png(filename=paste0(outfileDirectory,"preAlign_SG_SEURAT_RUN_ID.png"))
plot_grid(p1, p2)
dev.off()

png(filename=paste0(outfileDirectory,"preCCA_SG_SEURAT_RUN_ID.png"))
DimHeatmap(object = pbmc, reduction.type = "cca", cells.use = numCells, dim.use = 1:SG_DIMENSIONS, 
    do.balanced = TRUE)
dev.off()

writeLines("Calculating variance expression")

pbmc <- CalcVarExpRatio(object = pbmc, reduction.type = "pca", grouping.var = "protocol", 
    dims.use = 1:SG_DIMENSIONS)

writeLines("Aligning subspaces to SG_DIMENSIONS")

pbmc <- AlignSubspace(object = pbmc, reduction.type = "cca", grouping.var = "protocol", 
    dims.align = 1:SG_DIMENSIONS)

p1 <- VlnPlot(object = pbmc, features.plot = "ACC1", group.by = "protocol", 
              do.return = TRUE)
p2 <- VlnPlot(object = pbmc, features.plot = "ACC2", group.by = "protocol", 
              do.return = TRUE)

writeLines("Finished aligning subspaces; write violin plot")

png(filename=paste0(outfileDirectory,"aligned_SG_SEURAT_RUN_ID.png"))
plot_grid(p1, p2)
dev.off()

writeLines(" Finished: Creating violinplot of nUMI and nGene")
writeLines("Write Aligned files")

raw = as.matrix(x = pbmc@data)
exportDimReduxClustering = as.data.frame(raw[rownames(importedCSVfile),])
exportDimReduxClustering = exportDimReduxClustering[,colnames(importedCSVfile)]

exportDimReduxClustering2 = as.data.frame(raw[rownames(importedCSVfile2),])
exportDimReduxClustering2 = exportDimReduxClustering2[,colnames(importedCSVfile2)]

writeLines("Define file names")
header = ""
fileName = paste0(outfileDirectory,"Aligned_SG_SEURAT_RUN_ID_SG_TARGET_NAME_vs_SG_SAMPLE_NAME.csv")
fileName2 = paste0(outfileDirectory,"Aligned_SG_SEURAT_RUN_ID_SG_SAMPLE_NAME_vs_SG_TARGET_NAME.csv")

write.csv(header , file = fileName)
write.csv(header , file = fileName2)

header = "[Metadata]"
write.table(header, file = fileName, sep = ",", append = FALSE, quote = FALSE,
  col.names = FALSE, row.names = FALSE)
write.table(header, file = fileName2, sep = ",", append = FALSE, quote = FALSE,
  col.names = FALSE, row.names = FALSE)

header1 = c("$DataType , Expression")
write(header1,file=fileName, append=TRUE)
write(header1,file=fileName2, append=TRUE)

header2 = c("$Organization , ParamsInRows")
write(header2,file=fileName, append=TRUE)
write(header2,file=fileName2, append=TRUE)

header3 = ""
write(header3,file=fileName, append=TRUE)
write(header3,file=fileName2, append=TRUE)

header4 = "[Data]"
write(header4,file=fileName, append=TRUE)
write(header4,file=fileName2, append=TRUE)

writeLines("Write header and append expression matrix")

rowNamesDimRedux <- as.data.frame(row.names(exportDimReduxClustering))
colnames(rowNamesDimRedux) <- "Gene"
writeLines("Add CellId cell")

rowNamesDimRedux[,1] <- gsub("\\.", "-", rowNamesDimRedux[,1])
writeLines("Cleanup changes by R")

exportDimReduxClustering <- cbind(rowNamesDimRedux, exportDimReduxClustering)
is.num <- sapply(exportDimReduxClustering, is.numeric)
exportDimReduxClustering[is.num] <- lapply(exportDimReduxClustering[is.num], round, 2)
colnames(exportDimReduxClustering)[1] <- "Gene"
writeLines("Cleanup changes by R")

colnames(exportDimReduxClustering) <- gsub("AppID_*", "", colnames(exportDimReduxClustering))

write.table(as.matrix(exportDimReduxClustering), fileName, sep = ",", append = T, quote = FALSE, row.names = FALSE)
if(SG_IMPORT_KEEP_TARGET)
{
    write.table(as.matrix(targetKeepDF), fileName, sep = ",", append = T, quote = FALSE, col.names = FALSE)
}
rowNamesDimRedux2 <- as.data.frame(row.names(exportDimReduxClustering2))
rowNamesDimRedux2[,1] <- gsub("\\.", "-", rowNamesDimRedux2[,1])
colnames(rowNamesDimRedux2) <- "Gene"
exportDimReduxClustering2 <- cbind(rowNamesDimRedux2, exportDimReduxClustering2)
is.num <- sapply(exportDimReduxClustering2, is.numeric)
exportDimReduxClustering2[is.num] <- lapply(exportDimReduxClustering2[is.num], round, 2)
colnames(exportDimReduxClustering2)[1] <- "Gene"
colnames(exportDimReduxClustering2) <- gsub("AttID_*", "", colnames(exportDimReduxClustering2))

write.table(as.matrix(exportDimReduxClustering2), fileName2, sep = ",", append = T, quote = FALSE, row.names = FALSE)
if(SG_IMPORT_KEEP_SAMPLE)
{
    write.table(as.matrix(sampleKeepDF), fileName2, sep = ",", append = T, quote = FALSE, col.names = FALSE)
}

writeLines("Completed writting files, return to SeqGeq.")
