# SeqGeq / Seurat analysis

# Script modified from Satija lab's work for the R Seurat package.
# Script modified by Miguel Velazquez and Ian Taylor at FlowJo 
# For use w/ SeqGeq 
# Seurat is developed and maintained by the Satija lab. 
# Code is  released in GPL (3.0) license. 

#Disable error messaging in v1.0, prevent early exit.
options(warn=-1)


library(Seurat)
library(Matrix)
library(dplyr)


# Read CSV from SeqGeq
importedCSVfile <- read.table(file= "SG_SAMPLE_DATA_PATH", sep = ",", row.names = 1, header = TRUE)

# Some data has numbers as cellIDs only, append a string to cellIds, remove on export.
rownames(importedCSVfile) <- paste("AppID", rownames(importedCSVfile),sep="_")

# Set output directory
outfileDirectory = "SG_OUTPUT_DIRECTORY_PATH"

## From plugin, setup flow logic and save preferences to report.
runQC = SG_RUN_QC_BOOL
# Inside QC 
minCellCutoffBool = SG_CELL_QC_BOOL
minCellPercent = SG_MIN_CELL_BOOL
# minCellCutoff child
minGeneCutoffBool = SG_GENE_QC_BOOL
logNormalizeBool = SG_LOG_NORMALIZE_BOOL
saveObject = SG_SAVE_OBJECT_BOOL

# Run QC if not already run in SeqGeq or again.
if(runQC)
{
	writeLines(" Run Quality control")
	# Boolean for include genes w/ min detected expression
	if(minCellCutoffBool)
	{
		writeLines(" Minimum cell percent percent applied")
		minCellPercentVal = 0.05
		if(minCellPercent)
		{
			minCellPercentVal = SG_CELL_QC_PERCENT
		}
		minCellCutoff = nrow(importedCSVfile) * minCellPercentVal
	}
	minGeneCutoff = ncol(importedCSVfile) * 0.10
	if(minGeneCutoffBool)
	{
		minGeneCutoff = ncol(importedCSVfile) * SG_MIN_GENE_PERCENT
	}
	if(logNormalizeBool)
	{
		logNormalizeData = "SG_LOG_NORMALIZE_DATA"
	}
	writeLines(" Start: Create Seurat Object w/ QC")
	seuratSeqGeq <- CreateSeuratObject(
			raw.data = t(importedCSVfile), 
			min.cells = minCellCutoff, 
			min.genes = minGeneCutoff)
	rm(importedCSVfile)
	writeLines(" Finished: Creating Seurat object with QC")
}else{
	writeLines(" Start: Create Seurat Object w/ vanilla params")
	# No QC, just create Seurat object
	seuratSeqGeq <- CreateSeuratObject(
			raw.data = t(importedCSVfile))
	rm(importedCSVfile)
	writeLines(" Finished: Creating Seurat object")
}

filterCells = SG_FILTER_CELLS_BOOL
if(filterCells)
{
	writeLines(" Start: Filtering cells")
	subsetList = c("nGene")
	if(checkMitoGenes)
	{
		subsetList = append(subsetList, "percent.mito")
	}
	
	#thresholdSample = FALSE
	#Default thresholds
	lowthresholdValues = c(100, -Inf)
	highthresholdValues = c(2500, 0.05)
	thresholdSample = TRUE
	SG_LOW_THRESHOLD_VALUE = 100
	SG_HIGH_THRESHOLD_VALUE = 2500
	if(thresholdSample)
	{
		if(is.null(SG_LOW_THRESHOLD_VALUE) | is.null(SG_HIGH_THRESHOLD_VALUE))
		{
			lowthresholdValues = c(100, -Inf)
			highthresholdValues = c(2500, 0.05)
		}
		lowthresholdValues = c(SG_LOW_THRESHOLD_VALUE, -Inf)
		highthresholdValues = c(SG_HIGH_THRESHOLD_VALUE, 0.05)
	}
	seuratSeqGeq <- FilterCells(object = seuratSeqGeq, subset.names = subsetList, 
			low.thresholds = lowthresholdValues, high.thresholds = highthresholdValues)
	writeLines(" Finished: Filtering cells")
}

# Saves R rda object for later iterations
if(saveObject)
{
	writeLines(" Start: Save object selected")
	save(seuratSeqGeq, file = paste0(outfileDirectory,"SeqGeq_Seurat_Object.rda"), compress = TRUE)
	writeLines(" Finished: Saving RDA R object to directory")
}
#cat(row.names(as.data.frame(seuratSeqGeq@data)))

checkMitoGenes = TRUE
if(checkMitoGenes)
{
	writeLines(" Start: Checking for mitochondrial genes")
	writeLines(" Check for human MT mitochondrial genes")
	mito.genes <- grep(pattern = "^MT", x = rownames(x = seuratSeqGeq@data), value = TRUE)
	# Possibly mouse 
	if(length(mito.genes) < 1)
	{
		writeLines(" Potentially murine, check mt mitochondrial genes")
		mito.genes <- grep(pattern = "^mt", x = rownames(x = seuratSeqGeq@data), value = TRUE)
	}
	percent.mito <- Matrix::colSums(seuratSeqGeq@raw.data[mito.genes, ])/Matrix::colSums(seuratSeqGeq@raw.data)
	# AddMetaData adds columns to object@meta.data, and is a great place to
	# stash QC stats
	seuratSeqGeq <- AddMetaData(object = seuratSeqGeq, metadata = percent.mito, col.name = "percent.mito")
	
	# png(filename="QC_GENE_UMI_%_VLN_PLOT_PATH")
	if(length(mito.genes) > 1)
	{
		
		writeLines(" Creating violin plot of percent mitochondrial")
		#png(filename=paste0(outfileDirectory,"percentMito.png"))
		#VlnPlot(object = seuratSeqGeq, features.plot = c( "percent.mito" ), nCol = 1)
		#dev.off()
	}
	cat("Finished: Checking for mitochondrial genes")
}

# Log normalize data, add scaling factor
if(logNormalizeBool)
{
	writeLines(" Start: Log normalize matrix file")
	scalingFactor = 10000
	adjustScalingFactor = FALSE
	if(adjustScalingFactor)
	{
		scalingFactor = SG_SCALING_FACTOR
	}
	
	# Normalize data to scaling factor and method, def: LogNorm. TODO: Check for future methods
	seuratSeqGeq <- NormalizeData(object = seuratSeqGeq, normalization.method = "LogNormalize", 
			scale.factor = scalingFactor)
	exportNormData = FALSE
	if(exportNormData)
	{
		exportNormalizedData <- as.data.frame(seuratSeqGeq@scale.data)
	}
	writeLines(" Finished: Log normalizing data matrix")
	
}

# Find variable genes
findVarGenes = TRUE
if(findVarGenes)
{
	writeLines(" Start: Find variable genes")
	# Find variables genes and add to Seurat object
	seuratSeqGeq <- FindVariableGenes(object = seuratSeqGeq, mean.function = ExpMean, dispersion.function = LogVMR, 
			x.low.cutoff = 0.0125, x.high.cutoff = 8, y.cutoff = 0.5, display.progress = FALSE ,do.plot = FALSE)

	# Declare minimum genes to find, default is 10, can be more.
	minimumVarGenes = 10
	minVarGeneBool = FALSE
	if(minVarGeneBool)
	{
		minimumVarGenes = SG_MINIMUM_VAR_GENES
	}
	if(length(seuratSeqGeq@var.genes) < minimumVarGenes)
	{
		
		seuratSeqGeq <- FindVariableGenes(object = seuratSeqGeq, mean.function = ExpMean, dispersion.function = LogVMR,display.progress = FALSE ,do.plot = FALSE)
	}
	writeLines(" Finished: Finding variable genes")
}

regressOutVars = TRUE
if(regressOutVars)
{
	variablesToRegress = c("nUMI", "percent.mito")
	addMoreVarsRegress = FALSE
	if(addMoreVarsRegress)
	{
		#TODO: Make it a loop from a list defined in the plugin
		variablesToRegress = append(variablesToRegress, "SG_VARIABLES_TO_REGRESS")
	}
	seuratSeqGeq <- ScaleData(object = seuratSeqGeq, vars.to.regress = variablesToRegress)

}
runDimRedux = TRUE
#if(runDimRedux)
#{
# Run PCA w/ variables genes
runPCA = TRUE
if(runPCA)
{
	writeLines(" Start: Calculate PCA")
	seuratSeqGeq <- RunPCA(
			object = seuratSeqGeq, 
			pc.genes = seuratSeqGeq@var.genes)
	
	# Output a PCA to PNG
	#writeLines(" Created PCA plot")
	##png(filename=paste0(outfileDirectory,"pcaPlot.png"))
	#PCAPlot(object = seuratSeqGeq, dim.1 = 1, dim.2 = 2)
	#dev.off()
	
	# ProjectPCA scores each gene in the dataset (including genes not included
	# in the PCA) based on their correlation with the calculated components.
	seuratSeqGeq <- ProjectPCA(object = seuratSeqGeq, do.print = FALSE)
	writeLines(" Projecting PCA components")
	# Write dataframe PCA embeddings
	exportPCA <- seuratSeqGeq@dr$pca@cell.embeddings
	writeLines(" Finished: Calculating PCA components")
}

#Find Clusters and perform t-SNE w/ PCA
runTSNE = TRUE
if(runTSNE)
{
	writeLines(" Start: Find Clusters")
	
	seuratSeqGeq <- FindClusters(object = seuratSeqGeq, reduction.type = "pca", dims.use = 1:6, 
			resolution = 0.6, print.output = 0, save.SNN = TRUE, force.recalc = TRUE)
	
	exportKNNClusters <- as.data.frame(seuratSeqGeq@meta.data$res.0.6)
	colnames(exportKNNClusters) <- c("Seurat_Clusters")
	
	changePerplexity = SG_CHANGE_PERPLEXITY
	perplexityVal = 20
	if(changePerplexity && is.numeric(perplexityVal))
	{
		perplexityVal = SG_TSNE_PERPLEXITY
	}
	writeLines(" Start: Running t-SNE")
	
	# Run rTSNE in Seurat wrapper, use deterministic, assign perplexty
	seuratSeqGeq <- RunTSNE(object = seuratSeqGeq,check_duplicates = FALSE, seed.use = 1, perplexity = perplexityVal, dims.use = 1:10, do.fast = TRUE)
	writeLines(" Finished: Running t-SNE")
	exportTSNE <- seuratSeqGeq@dr$tsne@cell.embeddings
        colnames(exportTSNE) <- c("tSNE-X","tSNE-Y");
	#writeLines(" Finished: saving t-SNE")
	#write.csv(exportTSNE, file = paste0(outfileDirectory,"Seurat_tSNE.csv"), quote = FALSE, row.names = FALSE)
	
	# note that you can set do.label=T to help label individual clusters
	#png(filename=paste0(outfileDirectory,"tSNE_Plot.png"))
	#TSNEPlot(object = seuratSeqGeq)
	#dev.off()
	#writeLines(" Finished: saving t-SNE plot")

	# Export t-SNE X and y parameter

	
}

findClusterGenes = TRUE
if(findClusterGenes && runTSNE)
{
	writeLines(" Start: Finding differentially expressed genes, may take a while...")
	findClusterMethod = "negbinom"
	changeFindClusterMethod = TRUE
	if(changeFindClusterMethod)
	{
		## Only allow "bimod", "roc", "t", , "tobit", "poisson", "negbinom", default is negbinom
		findClusterMethod = "SG_FIND_CLUSTER_METHOD"
	}
	
	## Run test to find top markers for each cluster found in KNN
	seuratSeqGeq.markers <- FindAllMarkers(object = seuratSeqGeq, only.pos = TRUE, min.pct = 0.25, 
			thresh.use = 0.25, test.use = findClusterMethod, do.print=TRUE, random.seed = FALSE )

	col_idx <- grep("gene", names(seuratSeqGeq.markers))
	seuratSeqGeq.markers <- seuratSeqGeq.markers[, c(col_idx, (1:ncol(seuratSeqGeq.markers))[-col_idx])]
	names(seuratSeqGeq.markers)
	writeLines(" Finished: Finding DE genes")
	
	# Put top genes before clusters
	wheresymbol <- which(names(seuratSeqGeq.markers)=="gene")
	whereCluster <- which(names(seuratSeqGeq.markers)=="cluster")

	writeLines(" Start: Saving DE genes")
	exportBiomarkers <- seuratSeqGeq.markers[,c(wheresymbol,whereCluster)]
	write.csv(exportBiomarkers, file = paste0(outfileDirectory,"SG_SEURAT_RUN_ID","_Seurat_geneSets.csv"), quote = FALSE, row.names = FALSE)
	write.csv(seuratSeqGeq.markers, file = paste0(outfileDirectory,findClusterMethod,"_Summary.csv"),quote = FALSE, row.names = FALSE)
	writeLines(" Finished: Saving DE genes")
	
	findTop10Markers = FALSE
	# Allow reduction to top 10 for preview or redux mode.
	if(findTop10Markers)
	{
		top10 <- seuratSeqGeq.markers %>% group_by(cluster) %>% top_n(10, avg_diff)
		write.csv(top10, file = paste0(outfileDirectory,findClusterMethod,"_Top10",".csv"), quote = FALSE, row.names = FALSE)
	}
	writeLines(" Finished: Finding clusters")
}

writeLines(" Start: Outfile to merge.")
exportDimReduxClustering = cbind(exportPCA, exportTSNE, exportKNNClusters)
rowNamesDimRedux <- as.data.frame(row.names(exportDimReduxClustering))
colnames(rowNamesDimRedux) <- "CellId"
exportDimReduxClustering <- cbind(rowNamesDimRedux, exportDimReduxClustering)
is.num <- sapply(exportDimReduxClustering, is.numeric)
exportDimReduxClustering[is.num] <- lapply(exportDimReduxClustering[is.num], round, 2)
colnames(exportDimReduxClustering) <- paste( colnames(exportDimReduxClustering),"SG_SEURAT_RUN_ID",sep="_")
colnames(exportDimReduxClustering)[1] <- "CellId"
exportDimReduxClustering[,1] <- gsub("AppID_*", "", rownames(exportDimReduxClustering))

write.csv(exportDimReduxClustering, file = paste0(outfileDirectory,"PCA_tSNE_Clustering.csv"), row.names = FALSE, quote = FALSE)
writeLines(" Finished: Running t-SNE and outfile to merge.")
#}

exportALLDataBool = FALSE
if(exportALLDataBool)
{
	writeLines(" Start: Write out file containing data matrix and parameters")
	exportAllData <- cbind(t(as.data.frame(as.matrix(seuratSeqGeq@data))), exportDimReduxClustering)
	write.csv(exportAllData, file = paste0(outfileDirectory,"All_ScaledData",".csv"), quote = FALSE, row.names = FALSE)
}


# Default, Seurat calculates nGene and nUMI
#writeLines(" Start: Create violinplot of nUMI and nGene")

    png(filename=paste0(outfileDirectory,"qcPlot.png"))
    VlnPlot(object = seuratSeqGeq, features.plot = c( "nUMI", "nGene"), nCol = 2)

    dev.off()
writeLines(" Finished: Creating violinplot of nUMI and nGene")

#Clean up
sessionInfo()
writeLines(capture.output(sessionInfo()), paste0(outfileDirectory,"RsessionInfo.txt"))
writeLines(" Finished: Script, cleaning and writing out session info. Returning results back to SeqGeq.")

#print(" Saved image")
#save(seuratSeqGeq, file = paste0(outfileDirectory,"SeqGeq_Seurat_Object.rda"), compress = TRUE)
